#!/usr/bin/python
import vcs, build_system, imp
import argparse, os, sys, subprocess
from collections import OrderedDict

# read command-line arguments
parser = argparse.ArgumentParser(description='Build KDE')
parser.add_argument("-c, --config",
                    dest="config", default="config.py",
                    help="kdebuildpy config file")
parser.add_argument("--reconfigure",
                    action="store_true", dest="reconfigure",
                    help="Force configuration to be run")
parser.add_argument("--phases", choices=["update", "configure", "compile"], nargs='*', metavar='PHASE',
                    dest="phases", default=["update", "configure", "compile"],
                    help="For each module, run the given phases in the given order. Possible phases are: update, configure, compile")
parser.add_argument("--resume-from", metavar='MODULE',
                    dest="resume_from",
                    help="Resume building from the given repository")
parser.add_argument("modules",  metavar='MODULE', nargs='*',
                    help="Manually specify modules to be built")
args = parser.parse_args()

# load config
config = imp.load_source('config', args.config).__dict__
os.remove(args.config+'c') # remove compiled python file
projects = OrderedDict() # all projects
workProjects = [] # projects we work on

# an entire Project
class Project:
	def __init__(self, config, folder, module):
		self.folder = folder
		self.name = module['name']
		# VCS
		vcsName = module.get('vcs', 'kde+git')
		if vcsName == 'kde+git':
			self.vcs = vcs.KDEGit(self.sourceFolder(), module['name'], module['version'])
		elif vcsName == 'kde+svn':
			self.vcs = vcs.KDESVN(self.sourceFolder(), module['svn-path'])
		else:
			raise Exception("Unknown VCS type "+vcsName)
		# build system
		if config.get('buildDeb', False):
			self.buildSystem = build_system.AutoDebuild(self.sourceFolder(), self.buildFolder(), module, self.vcs, config)
		else:
			buildSystemName = module.get('build-system', 'cmake')
			if buildSystemName == 'cmake':
				self.buildSystem = build_system.CMake(self.sourceFolder(), self.buildFolder(), module, config)
			else:
				raise Exception("Unknown build system type "+buildSystemName)
	
	def sourceFolder(self):
		return os.path.join(self.folder, self.name)
	
	def buildFolder(self):
		return os.path.join(config['buildDir'], self.sourceFolder())

# return the position of the given item in the list
def findInList(list, item):
	for i in xrange(len(list)):
		if list[i] == item:
			return i
	raise Exception("%s not found in list" % str(item))

# populate list of projects
def loadProjects(config, modules, folder=''):
	for module in modules:
		if 'folder' in module: # a subpath
			loadProjects(config, module['modules'], os.path.join(folder, module['folder']))
		else: # a proper project
			if module['name'] in projects:
				raise Exception("Duplicate module name "+module['name'])
			projects[module['name']] = Project(config, folder, module)

# now check what we have to do
loadProjects(config, config['modules'])
if args.modules:
	if args.resume_from is not None:
		raise Exception("Can not use --resume-from and manually specify modules")
	for module in args.modules:
		if not module in projects:
			raise Exception("Project %s does not exist" % module)
		workProjects.append(projects[module])
elif args.resume_from is None:
	workProjects = projects.values() # all the projects
else:
	if not args.resume_from in projects:
		raise Exception("Project %s does not exist" % args.resume_from)
	startWith = projects[args.resume_from]
	startIndex = findInList(projects.values(), startWith)
	workProjects = projects.values()[startIndex:]

# and do it!
for project in workProjects:
	try:
		for phase in args.phases:
			if phase == 'update':
				print "Updating",project.sourceFolder()
				project.vcs.update()
			elif phase == 'configure':
				print "Configuring",project.sourceFolder()
				project.buildSystem.configure(force=args.reconfigure)
			elif phase == 'compile':
				print "Compiling",project.sourceFolder()
				project.buildSystem.build()
				print "Installing",project.sourceFolder()
				project.buildSystem.install()
			else:
				raise Exception("Invalid phase "+phase)
	except (subprocess.CalledProcessError, KeyboardInterrupt) as e:
		print >> sys.stderr
		print >> sys.stderr
		if isinstance(e, KeyboardInterrupt): # str(e) would be the empty string
			print >> sys.stderr, "Interruped by user while processing %s" % (project.name)
		else:
			print >> sys.stderr, "Error while processing %s: %s" % (project.name, str(e))
		print >> sys.stderr
		sys.exit(1)
